/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 *
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * AllShapesAction.cpp
 *
 *  Created on: Sep 14, 2012
 *      Author: ankele
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "Shapes/ShapeRegistry.hpp"

#include <boost/foreach.hpp>
#include <iostream>
#include <string>

#include "AllShapesAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "AllShapesAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr SelectionAllShapesAction::performCall() {
  std::vector<Shape *> selectedShapes= ShapeRegistry::getInstance().getSelectedShapes();
  LOG(1, "Selecting all shapes.");
  ShapeRegistry::getInstance().selectAllShapes();
  LOG(0, ShapeRegistry::getInstance().countSelectedShapes() << " shapes selected.");
  return ActionState::ptr(new SelectionAllShapesState(selectedShapes,params));
}

ActionState::ptr SelectionAllShapesAction::performUndo(ActionState::ptr _state) {
  SelectionAllShapesState *state = assert_cast<SelectionAllShapesState*>(_state.get());

  ShapeRegistry::getInstance().unselectAllShapes();

  BOOST_FOREACH(Shape *shape, state->selectedShapes)
  ShapeRegistry::getInstance().selectShape(shape);

  return ActionState::ptr(_state);
}

ActionState::ptr SelectionAllShapesAction::performRedo(ActionState::ptr _state){
  //SelectionAllShapesState *state = assert_cast<SelectionAllShapesState*>(_state.get());

  ShapeRegistry::getInstance().selectAllShapes();

  return ActionState::ptr(_state);
}

bool SelectionAllShapesAction::canUndo() {
  return true;
}

bool SelectionAllShapesAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
